@@ -72,7 +72,11 @@ static void gff_load_metrics (GnomeFontFace *face, gint glyph);
 static void gff_load_outline (GnomeFontFace *face, gint glyph);
 
 static void gf_pso_print_sized (GnomeFontPsObject *pso, const guchar *text, gint size);
-static void gf_pso_sprintf (GnomeFontPsObject * pso, const gchar * format, ...);
+static void gf_pso_sprintf (GnomeFontPsObject * pso, 
+			    const gchar * format, ...);
+static gint gf_pso_print_double (GnomeFontPsObject *pso, const gchar *format,
+				 gdouble x);
+
 static void gnome_font_face_ps_embed_ensure_size (GnomeFontPsObject * pso, gint size);
 
 #define GFE_IS_T1(e) ((e)->type == GP_FONT_ENTRY_TYPE1)
@@ -788,7 +792,7 @@ gff_load (GnomeFontFace *face)
 		entry = face->entry;
 	}
 
-	ft_result = FT_New_Face (ft_library, entry->file, 0, &ft_face);
+	ft_result = FT_New_Face (ft_library, entry->file, entry->index, &ft_face);
 	g_return_val_if_fail (ft_result == FT_Err_Ok, FALSE);
 
 	psname = FT_Get_Postscript_Name (ft_face);
@@ -1325,13 +1329,18 @@ gnome_font_face_ps_embed_tt (GnomeFontPsObject *pso)
 	embeddedname = pso->face->psname;
 
 	fbuf = b.buf;
-	strings = gp_tt_split_file (fbuf, b.buf_size);
+	strings = gp_tt_split_file (fbuf, b.buf_size, pso->face->entry->index);
 	if (strings) {
 		/* Relatively easy - just embed TTF into PS stream */
 		/* Download master font */
 		TTVersion = 1.0;
 		MfrRevision = 1.0;
-		gf_pso_sprintf (pso, "%%!PS-TrueTypeFont-%g-%g\n", TTVersion, MfrRevision);
+		gf_pso_sprintf (pso, "%%!PS-TrueTypeFont-");
+		gf_pso_print_double (pso, "%g", TTVersion);
+		gf_pso_sprintf (pso, "-");
+		gf_pso_print_double (pso, "%g", MfrRevision);
+		gf_pso_sprintf (pso, "\n");
+
 		gf_pso_sprintf (pso, "11 dict begin\n");
 		gf_pso_sprintf (pso, "/FontName (%s) cvn def\n", embeddedname);
 		gf_pso_sprintf (pso, "/Encoding 256 array\n");
@@ -1341,7 +1350,15 @@ gnome_font_face_ps_embed_tt (GnomeFontPsObject *pso)
 		gf_pso_sprintf (pso, "/FontMatrix [1 0 0 1 0 0] def\n");
 		/* fixme: */
 		bbox = gnome_font_face_get_stdbbox (pso->face);
-		gf_pso_sprintf (pso, "/FontBBox [%g %g %g %g] def\n", bbox->x0, bbox->y0, bbox->x1, bbox->y1);
+		gf_pso_sprintf (pso, "/FontBBox [");
+		gf_pso_print_double (pso, "%g",  bbox->x0);
+		gf_pso_sprintf (pso, " ");
+		gf_pso_print_double (pso, "%g", bbox->y0);
+		gf_pso_sprintf (pso, " ");
+		gf_pso_print_double (pso, "%g",  bbox->x1);
+		gf_pso_sprintf (pso, " ");
+		gf_pso_print_double (pso, "%g",  bbox->y1);
+		gf_pso_sprintf (pso, "] def\n");
 		gf_pso_sprintf (pso, "/FontType 42 def\n");
 		/* fixme: XUID */
 		/* fixme: Be more intelligent */
@@ -1516,30 +1533,38 @@ gf_pso_print_sized (GnomeFontPsObject *pso, const guchar *text, gint size)
 	pso->length += size;
 }
 
+/* Note "format" should be locale independent, so it should not use %g */
+/* and friends */
 static void
 gf_pso_sprintf (GnomeFontPsObject *pso, const gchar * format, ...)
 {
 	va_list arguments;
-	gchar *oldlocale;
 	gchar *text;
-	gint len;
 	
-	oldlocale = g_strdup (setlocale (LC_NUMERIC, NULL));
-	setlocale (LC_NUMERIC, "C");
-#warning: We should not be using setlocale
-		
 	va_start (arguments, format);
 	text = g_strdup_vprintf (format, arguments);
 	va_end (arguments);
 
-	len = strlen (text);
-	gf_pso_print_sized (pso, text, len);
+	gf_pso_print_sized (pso, text, strlen (text));
+	g_free (text);
+}
+
+/* Allowed conversion specifiers are 'e', 'E', 'f', 'F', 'g' and 'G'. */
+static gint   
+gf_pso_print_double (GnomeFontPsObject *pso, const gchar *format, gdouble x)
+{
+ 	gchar *text;
+
+	text = g_new (gchar, G_ASCII_DTOSTR_BUF_SIZE);
+	g_ascii_formatd (text, G_ASCII_DTOSTR_BUF_SIZE, format, x);
+
+	gf_pso_print_sized (pso, text, strlen (text));
 	g_free (text);
 
-	setlocale (LC_NUMERIC, oldlocale);
-	g_free (oldlocale);
+	return GNOME_PRINT_OK;
 }
 
+
 static void
 gnome_font_face_ps_embed_ensure_size (GnomeFontPsObject *pso, gint size)
 {
