@@ -152,7 +152,7 @@ fcpattern_to_gp_font_entry (FcPattern *font)
 	GPFontEntry *e = NULL;
 	FcResult result;
 	FcChar8 *fc_family, *fc_style, *fc_file;
-	int italic_angle, weight;
+	int italic_angle, weight, face_index;
 	gboolean outline;
 	gchar *c;
 
@@ -178,6 +178,9 @@ fcpattern_to_gp_font_entry (FcPattern *font)
 		return NULL;
 	}
 
+	face_index = 0;
+	result = FcPatternGetInteger (font, FC_INDEX, 0, &face_index);
+
 	italic_angle = 0;
 	FcPatternGetInteger (font, FC_SLANT, 0, &italic_angle);
 
@@ -203,6 +206,7 @@ fcpattern_to_gp_font_entry (FcPattern *font)
 	e = g_new0 (GPFontEntry, 1);
 	e->type         = font_type;
 	e->file         = g_strdup (fc_file);
+	e->index        = face_index;
 	e->refcount     = 1;
 	e->face         = NULL;
 	e->speciesname  = g_strdup (fc_style);
