@@ -518,7 +518,7 @@ gnome_print_job_print (GnomePrintJob *job)
 	JOBPrivate *pp;
 	GnomePrintContext *ctx;
 	gint lpages, copies, nstacks, npages, nsheets;
-	gboolean collate, is_multipage;
+	gboolean collate, is_multipage, hwcopies;
 	gint stack;
 	const guchar *buf;
 	gint blen;
@@ -573,6 +573,15 @@ gnome_print_job_print (GnomePrintJob *job)
 	gnome_print_config_get_boolean (job->config, GNOME_PRINT_KEY_COLLATE, &collate);
 	copies = 1;
 	gnome_print_config_get_int (job->config, GNOME_PRINT_KEY_NUM_COPIES, &copies);
+	hwcopies = FALSE;
+	gnome_print_config_get_boolean (job->config, 
+					collate 
+					? GNOME_PRINT_KEY_COLLATED_COPIES_IN_HW
+					: GNOME_PRINT_KEY_NONCOLLATED_COPIES_IN_HW, 
+					&hwcopies);
+
+	if (hwcopies)
+		copies = 1;
 
 	if (collate) {
 		nstacks = copies;
@@ -603,6 +612,10 @@ gnome_print_job_print (GnomePrintJob *job)
 				}
 			}
 		}
+		if (stack + 1 < nstacks) {
+			ret = gnome_print_end_doc (ctx);
+			g_return_val_if_fail (ret == GNOME_PRINT_OK, ret);
+		}
 	}
 
 	ret = gnome_print_context_close (ctx);
