@@ -29,12 +29,13 @@
 
 #define GUINT32FROMPTR(p) ((*(guchar*)((p))<<24)|(*((guchar*)(p)+1)<<16)|(*((guchar*)(p)+2)<<8)|(*((guchar*)(p)+3)))
 #define GUINT16FROMPTR(p) ((*(guchar*)((p))<<8)|(*((guchar*)(p)+1)))
+#define GP_TT_IS_TTC(b) (!strncmp (b, "ttcf", 4))
 
 static GSList *gp_tt_split_glyf (const guchar *buf, guint glyf, guint next, guint loca, guint head, guint maxp, GSList *slices);
 static gint gp_tt_offset_compare (gconstpointer a, gconstpointer b);
 
 GSList *
-gp_tt_split_file (const guchar *buf, guint len)
+gp_tt_split_file (const guchar *buf, guint len, guint face)
 {
 	const guchar *b;
 	GSList *tables;
@@ -48,25 +49,42 @@ gp_tt_split_file (const guchar *buf, guint len)
 	guint16 entrySelector;
 	guint16 rangeShift;
 
-	guint32 glyf, loca, head, maxp;
+	guint32 glyf, loca, head, maxp, offset = 0;
 
 	b = buf;
 	tables = NULL;
 
+	if (GP_TT_IS_TTC (b)) {
+		guint32 version, num_fonts;
+
+		/* this file is TrueType Collection file */
+		version = GUINT32FROMPTR (b + 4);
+		if (version >= 0x00010000) {
+			num_fonts = GUINT32FROMPTR (b + 8);
+			if (face > num_fonts) {
+				g_warning ("font doesn't have a face %d. number of face is %d.", face, num_fonts);
+				return NULL;
+			}
+			offset = GUINT32FROMPTR (b + 12 + face * 4);
+		} else {
+			g_warning ("unknown TTC version %X", version);
+			return NULL;
+		}
+	}
 	/* Font Directory */
 	/* Offset Subtable */
 
-	scaler_type   = GUINT32FROMPTR (b);
-	numTables     = GUINT16FROMPTR (b + 4);
-	searchRange   = GUINT16FROMPTR (b + 6);
-	entrySelector = GUINT16FROMPTR (b + 8);
-	rangeShift    = GUINT16FROMPTR (b + 10);
+	scaler_type   = GUINT32FROMPTR (b + offset);
+	numTables     = GUINT16FROMPTR (b + offset + 4);
+	searchRange   = GUINT16FROMPTR (b + offset + 6);
+	entrySelector = GUINT16FROMPTR (b + offset + 8);
+	rangeShift    = GUINT16FROMPTR (b + offset + 10);
 
 	if ((scaler_type != 0x74727565) && (scaler_type != 0x00010000))
 		return NULL;
 
 	/* Table Directory */
-	b = buf + 12;
+	b = buf + offset + 12;
 	glyf = 0;
 	loca = 0;
 	head = 0;
