@@ -35,10 +35,28 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <unistd.h>
+#include <locale.h>
 
 #include <cups/cups.h>
 #include <libgnomeprint/gnome-print.h>
 #include <libgnomeprint/gnome-print-transport.h>
+#include <libgnomeprint/gnome-print-config.h>
+
+/* We are protecting the locale due to a known bug in the libcups library. */
+#define LOCALEPROTECTION(c) { char *OLDLOCALE;\
+                              OLDLOCALE = g_strdup(setlocale(LC_ALL, NULL)); \
+                              c; \
+                              if (strcmp (OLDLOCALE, setlocale (LC_ALL, NULL))) {\
+                                       static gboolean LOCALEPROTECTIONWARNING = FALSE;\
+                                       if (!LOCALEPROTECTIONWARNING) {\
+                                                LOCALEPROTECTIONWARNING = TRUE;\
+                                                g_warning ("Working around locale bug in cups.");\
+                                       }\
+                                       setlocale (LC_ALL, OLDLOCALE);\
+			      }\
+                              g_free (OLDLOCALE);\
+                            }
+
 
 #define GP_TYPE_TRANSPORT_CUPS         (gp_transport_cups_get_type ())
 #define GP_TRANSPORT_CUPS(o)           (G_TYPE_CHECK_INSTANCE_CAST ((o), GP_TYPE_TRANSPORT_CUPS, GPTransportCups))
@@ -204,10 +222,22 @@ get_job_options (GnomePrintConfig *config, cups_option_t **options)
 	gchar *value;
 	gint num = 0;
 
-	value = gnome_print_config_get (config, "Settings.Output.PaperSource");
-	num = cupsAddOption ("InputSlot", value, num, options);
-	g_free (value);
+	value = gnome_print_config_get (config, GNOME_PRINT_KEY_PAPER_SOURCE);
+
+	if (value != NULL) {
+		LOCALEPROTECTION (num = cupsAddOption 
+				  ("InputSlot", value, num, options))
+			g_free (value);
+	}
 	
+	value = gnome_print_config_get (config, GNOME_PRINT_KEY_HOLD);
+
+	if (value != NULL) {
+		LOCALEPROTECTION (num = cupsAddOption 
+				  ("job-hold-until", value, num, options))
+			g_free (value);
+	}
+
 	return num;
 }
 	
@@ -234,9 +264,11 @@ gp_transport_cups_close (GnomePrintTransport *gp_transport)
 					GNOME_PRINT_KEY_DOCUMENT_NAME);
 	num_options = get_job_options (gp_transport->config, &options);
 	
-	cupsPrintFile (transport->printer, transport->temp_file, title, num_options, options);
+	LOCALEPROTECTION (cupsPrintFile (transport->printer, 
+					 transport->temp_file, 
+					 title, num_options, options);
+			  cupsFreeOptions (num_options, options))
 
-	cupsFreeOptions (num_options, options);
 	unlink (transport->temp_file);
 	g_free (title);
 	
@@ -283,9 +315,11 @@ gp_transport_cups_print_file (GnomePrintTransport *gp_transport, const guchar *f
 					GNOME_PRINT_KEY_DOCUMENT_NAME);
 	num_options = get_job_options (gp_transport->config, &options);
 	
-	cupsPrintFile (transport->printer, filename, title, num_options, options);
+	LOCALEPROTECTION(cupsPrintFile (transport->printer, 
+					filename, title, 
+					num_options, options);
+			 cupsFreeOptions (num_options, options))
 
-	cupsFreeOptions (num_options, options);
 	g_free (title);
 	
 	return GNOME_PRINT_OK;
